#! /usr/bin/env python
#
# Copyright 2007  Jochen Voss <voss@seehuhn.de>
#
# This file is part of the CSimGen package.
#
# CSimGen is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""Helper functions.

WARNING: this is an experimental version of CSimGen.  The API is not
yet finalised and may change without further notice.
"""

__author__ = 'Jochen Voss'
__version__ = '0.1'

from value import value

######################################################################
# optinally generated auxiliary functions

def new_matrix_from_file_auxfn(source):
    fn = source.function("new_matrix_from_file",
                         ("fname", "const char", 0),
                         ("m_ret", "int", 0),
                         ("n_ret", "int", 0),
                         rtype = "static double *")
    data = value("data")
    fn.add_definition(data, "double", None, 0)
    allocated = fn.define_int(64, name="allocated")
    fn.helper("malloc", data, "double", allocated)
    l = fn.define_int(0, name="l")
    m = fn.define_int(0, name="m")
    n = fn.define_int(0, name="n")

    fd = fn.define_file("fd", "fname")
    fn.add_include ("<stdlib.h>")
    fn.cond("! fd", "*m_ret = *n_ret = 0;", "return NULL;")
    err = fn.define_int(0, name="err")
    fn.add_include("<stdio.h>")
    loop = fn.while_loop("! feof (fd)")
    c = loop.define_int("getc (fd)", name="c")

    loop.cond ("c == EOF", "break;")
    i = loop.cond (r"c == '\n'")
    i.cond("l == 0", "break;")
    i.cond("n == 0", "n = l;")
    i.cond("l != n", "err = 1;", "break;")
    i.statement("++m;",
                "l=0;",
                "continue;")
    loop.add_include("<ctype.h>")
    loop.cond ("isspace(c)", "continue;")

    loop.statement("",
                   "ungetc (c, fd);")
    lc = loop.cond("m*n+l+1 > allocated")
    lc.statement("allocated *= 2;")
    lc.helper("realloc", data, "double", allocated)
    rc = loop.define_int(name="rc")
    loop.funcall("fscanf", fd, '"%lf"', data+m*n+l, res=rc)
    loop.cond("rc != 1", "err = 1;", "break;")
    loop.statement("++l;")
    fn.close_file(fd)

    i = fn.cond("l > 0")
    i.cond("n == 0", "n = l;")
    i.statement("if (l != n) err = 1; else ++m;")

    fc = fn.cond(err)
    fc.helper("error", "%s: malformed matrix", "fname")
    fn.helper("realloc", data, "double", m*n)

    fn.statement("*m_ret = m;",
                 "*n_ret = n;")
    fn.ret = "return  data;"
    return  fn

def compare_doubles_auxfn(source):
    fn = source.function("compare_doubles",
                         ("a", "const void", 0),
                         ("b", "const void", 0),
                         rtype = "int",
                         static = True)
    fn.add_definition ('aa','const double', 'a', 0)
    fn.add_definition ('bb','const double', 'b', 0)
    i1 = fn.cond ("*aa > *bb", "return 1;")
    i2 = fn.cond ("*aa < *bb", "return -1;")
    fn.statement ("return 0;")
    return  fn

def compare_doubles_reverse_auxfn(source):
    fn = source.function("compare_doubles_reverse",
                         ("a", "const void", 0),
                         ("b", "const void", 0),
                         rtype = "int",
                         static = True)
    fn.add_definition ('aa','const double', 'a', 0)
    fn.add_definition ('bb','const double', 'b', 0)
    i1 = fn.cond ("*aa < *bb", "return 1;")
    i2 = fn.cond ("*aa > *bb", "return -1;")
    fn.statement ("return 0;")
    return  fn

def get_auxfns():
    g = globals()
    marker = "_auxfn"
    res = {}
    for name in g:
        if not name.endswith(marker): continue
        fn = name[:-len(marker)]
        res[fn] = g[name]
    return  res

######################################################################
# default actions for configurable function calls

def malloc_helper(block, name, type, count):
    block.add_include('<stdlib.h>')
    block.funcall("malloc", count*value("sizeof(%s)"%type), res=name)

def realloc_helper(block, name, type, count):
    block.add_include('<stdlib.h>')
    block.funcall("realloc", name, count*value("sizeof(%s)"%type), res=name)

def free_helper(block, name, position="body"):
    block.add_include('<stdlib.h>')
    block.funcall("free", name, position=position)

def error_helper(block, message, *args):
    block.add_include('<stdio.h>')
    args = ["stderr"] + ['"error: %s\\n"'%message] + list(args)
    block.funcall("fprintf", *args)

def get_helpers():
    g = globals()
    marker = "_helper"
    res = {}
    for name in g:
        if not name.endswith(marker): continue
        fn = name[:-len(marker)]
        res[fn] = g[name]
    return res
